﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;
using Demo.ViewModels.Items;

namespace Demo.ViewModels
{
    public class SearchResultPageVm : VmBase
    {
        public ObservableCollection<SearchFilterVmi> Filters { get; set; }
        public SearchFilterVmi SelectedFilter { get; set; }
        public string QueryText { get; set; }

        public SearchResultPageVm()
        {
            if (DesignMode)
            {
                SampleData();
            }
        }

        public new async Task Init()
        {
            await base.Init();
        }

        private void SampleData()
        {
            var filters = new List<SearchFilterVmi>();

            var books = new List<BookVmi>
                {
                    new BookVmi
                        {
                            Title = "The Colour of Magic",
                            Cover = "/images/TCoM_cover.jpg",
                            Serie = "Rincewind",
                            Year = "1983",
                            Description =
                                "The Colour of Magic is a 1983 comic fantasy novel by Terry Pratchett, and is the first book of the Discworld series."
                        },
                    new BookVmi
                        {
                            Title = "The Colour of Magic",
                            Cover = "/images/TCoM_cover.jpg",
                            Serie = "Rincewind",
                            Year = "1983",
                            Description =
                                "The Colour of Magic is a 1983 comic fantasy novel by Terry Pratchett, and is the first book of the Discworld series."
                        },
                };

            filters.Add(new SearchFilterVmi { Active = true,Name="all",Books =books });
            filters.Add(new SearchFilterVmi { Active = false,Name="lorem",Books =books });
            filters.Add(new SearchFilterVmi { Active = false,Name="ipsum",Books =books });

            Filters = new ObservableCollection<SearchFilterVmi>(filters);
           
            SelectedFilter = Filters[0];
            QueryText = "sample test";
        }


        public void Search(string query)
        {
            var queryText = query.ToLower();

            var generator = new BookGenerator();
            var books = generator.CreateDiscWorldCollection();
            var filters = new List<SearchFilterVmi>();

            var result = books.FindAll(x => x.Title.ToLower().Contains(queryText) || x.Description.ToLower().Contains(queryText));

            if (result.Count > 0)
            {
                var all = new List<BookVmi>(result);
                var groupedBooks = result.GroupBy(x => x.Serie);

                filters.AddRange(from @group in groupedBooks
                                 let items = new List<BookVmi>(@group.ToList())
                                 select new SearchFilterVmi {Active = false, Books = items, Name = @group.Key});

                filters.Sort((x, y) => y.Books.Count.CompareTo(x.Books.Count));
                filters.Insert(0, new SearchFilterVmi {Active = true, Books = all, Name = "All"});

                Filters = new ObservableCollection<SearchFilterVmi>(filters);
                SelectedFilter = Filters[0];
            }
            else
            {
                Filters = new ObservableCollection<SearchFilterVmi>();
            }

            QueryText = queryText;
        }
    }
}